/*
 * aconversion
 *
 * Usage: aconversion [-V] -t target [-a algorithm] < input.xml
 *
 */

#include <iostream>
#include <getopt.h>

#include <sax/SaxInterface.h>
#include <sax/ParserException.h>

#include "ConversionHandler.h"

using namespace std;
using namespace sax;
using namespace alib;
using namespace conversions;


void help( void )
{
    cout << endl;
    cout << "aconversion 0.01" << endl;
    cout << "Converts between regular expressions, regular grammars and finite automata." << endl;
    cout << "Usage: aconversion -t FORMALISM [SWITCH...]" << endl << endl;
    cout << "  -t, --target=FORMALISM \t Specifies target formalism, valid values are 'FA', 'FSM' or 'automaton' for finite automata, 'RE' or 'regexp' for regular expression and 'RG' or 'grammar' for regular grammar." << endl;
    cout << "  -a, --algorithm=ALGORITHM \t Specifies algorithm to use. See --list-algorithms." << endl;
    cout << "  -l, --list-algorithms \t Lists algorithms to use." << endl;

    cout << endl;
}

void list_algorithms( void )
{
    cout << "List of available algorithms:" << endl;


    cout << " |" << endl;
    cout << " +- FA to RE:" << endl;
    cout << " | +- 'algebraic' - Converts to RE using Brzozowski algebraic method." << endl;
    cout << " | +- 'elimination' [Default] - Performs state elimination algorithm on NFA." << endl;

    cout << " +- FA to RG:" << endl;
    cout << " | +- 'left' - Converts to left regular grammar." << endl;
    cout << " | +- 'right' [Default] - Converts to right regular grammar." << endl;

    cout << " +- RE to FA:" << endl;
    cout << " | +- 'brzozowski' - Converts to right regular grammar." << endl;
    cout << " | +- 'thompson' - Converts to right regular grammar." << endl;
    cout << " | +- 'glushkov' [Default] - Converts to right regular grammar." << endl;

    cout << " +- RE to RG:" << endl;

    cout << " +- RG to FA:" << endl;
    cout << " | +- 'rg' [Default] - Converts right or left regular grammar to finite automata." << endl;

    cout << " +- RG to RE:" << endl;
}

int main(int argc, char* argv[])
{
    static struct option long_options[] = {
      {"target",            required_argument,  NULL, 't'},
      {"algorithm",         required_argument,  NULL, 'a'},
      {"version",           no_argument,        NULL, 'v'},
      {"help",              no_argument,        NULL, 'h'},
      {"list-algorithms",   no_argument,        NULL, 'l'},
      {0, 0, 0, 0}
    };

    int long_index = 0, opt = 0;
    string target, algorithm;

    while( ( opt = getopt_long( argc, argv,"a:t:vhl", long_options, & long_index ) )  != -1 )
    {
        switch( opt )
        {
        case 'a':
            algorithm.assign( optarg, strlen( optarg ) );
            break;
        case 't':
            target.assign( optarg, strlen( optarg ) );
            break;
        case 'l':
            list_algorithms( );
            return 0;
        case 'v':
        case 'h':
        default:
            help( );
            return 0;
        }
    }

  list<Token> tokens;
  string input( istreambuf_iterator<char>( cin ), ( istreambuf_iterator<char>( ) ) );
  SaxInterface::parseMemory( input, tokens );

  try
  {
      ConversionHandler conv( tokens, target, algorithm );
      conv.convert( cout );
  }
  catch( AlibException & e )
  {
      cout << e.what( ) << endl;
      return 1;
  }

  return 0;
}
