/*
 * BrzozowskiAlgebraic.h
 *
 *  Created on: 11. 2. 2014
 *      Author: tomas
 */

#ifndef BRZOZOWSKIALGEBRAIC_H_
#define BRZOZOWSKIALGEBRAIC_H_

#include <deque>
#include <map>
#include <queue>

#include <regexp/RegExpElements.h>

#include "AbstractFAtoREConverter.h"
#include "../shared/RightRegularEquationSolver.h"

namespace conversions
{

// http://cs.stackexchange.com/questions/2016/how-to-convert-finite-automata-to-regular-expressions
// https://github.com/ferno/greenery/blob/bcc0a136335edbe94cd7725fc6e8cce0268d850c/fsm.py

/**
 * Converts FA to RE using Brzozowski's algebraic method applied on right
 * regular equations.
 */
class BrzozowskiAlgebraic: public conversions::AbstractFAtoREConverter
{
public:
    BrzozowskiAlgebraic( const automaton::FSM & fsm );
    ~BrzozowskiAlgebraic( void );
    regexp::RegExp convert( void );

private:
    void initEquations( void );

    RightRegularEquationSolver m_solver;
};

} /* namespace conversions */

#endif /* BRZOZOWSKIALGEBRAIC_H_ */
