/*
 * StateElimination.h
 *
 *  Created on: 9. 2. 2014
 *      Author: tomas
 */

#ifndef STATEELIMINATION_H_
#define STATEELIMINATION_H_

#include <set>

#include <automaton/FSM/FSM.h>
#include <regexp/RegExp.h>
#include <regexp/RegExpElements.h>

#include "AbstractFAtoREConverter.h"

#include "RegExpOptimize.h"

namespace conversions
{

class StateElimination: public conversions::AbstractFAtoREConverter
{
public:
    StateElimination( const automaton::FSM & fsm );
    ~StateElimination( void );
    regexp::RegExp convert( void );

private:
    struct TransitionExtendedNFA
    {
        const automaton::State m_from, m_to;
        regexp::RegExpElement* m_regexp;

        TransitionExtendedNFA( const automaton::State & from, regexp::RegExpElement* regexp ,const automaton::State & to );
        bool operator<( const TransitionExtendedNFA & rhs ) const;
    };

    struct ExtendedNFA
    {
        std::set<automaton::State> m_states, m_finalStates, m_initialStates;
        std::set<alphabet::Symbol> m_alphabet;
        std::set<TransitionExtendedNFA> m_transitions;

        const automaton::State & createUniqueState( const std::string & name );
        std::set<StateElimination::TransitionExtendedNFA> transitionsToState( const automaton::State & toState ) const;
    };

    void constructExtendedNFA( void );
    void extendExtendedNFA( void );
    void eliminate( const automaton::State & q );
    void deleteTransitions( void );
    regexp::RegExpElement* transition( const automaton::State & p, const automaton::State & q ) const;

    ExtendedNFA m_MR;
};

} /* namespace conversions */

#endif /* STATEELIMINATION_H_ */
