/*
 * Thompson.h
 *
 *  Created on: 11. 1. 2014
 *      Author: tomas
 */

#ifndef THOMPSON_H_
#define THOMPSON_H_

#include <set>
#include <vector>

#include <automaton/FSM/FSM.h>
#include <regexp/RegExp.h>
#include <regexp/RegExpElements.h>

#include "AbstractREtoFAConverter.h"

#include "RegExpAlphabet.h"

namespace conversions
{

/**
 * Converts regular expression to finite automata using Thompson's Construction Algorithm (TCA).
 * Sources:
 *  Hopcroft, section 3.2.3
 *  http://www.eecis.udel.edu/~cavazos/cisc672/lectures/Lecture-04.pdf
 *  http://citeseerx.ist.psu.edu/viewdoc/download?doi=10.1.1.21.7450&rep=rep1&type=ps
 */
class Thompson : public AbstractREtoFAConverter
{
public:
    Thompson( const regexp::RegExp & re );
    ~Thompson( void );
    automaton::FSM convert( void );

private:
    /**
     * Stores head and tail state of "subautomaton" created in regexp subtree.
     */
    struct SubexpressionTails
    {
        SubexpressionTails( const automaton::State & head, const automaton::State & tail );
        automaton::State m_head, m_tail;
    };

    SubexpressionTails processRegExpNode( const regexp::RegExpElement * node );
    SubexpressionTails processRegExpNode( const regexp::Alternation * node );
    SubexpressionTails processRegExpNode( const regexp::Concatenation * node );
    SubexpressionTails processRegExpNode( const regexp::Iteration * node );
    SubexpressionTails processRegExpNode( const regexp::RegExpSymbol * node );
    SubexpressionTails processRegExpNode( const regexp::RegExpEmpty * node );
    SubexpressionTails processRegExpNode( const regexp::RegExpEpsilon * node );
};

} /* namespace conversions */

#endif /* THOMPSON_H_ */
